@@ -1,5 +1,5 @@
 /* task.c: Primitive functions for user-facing gzochi task management API
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2015 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 #include <libguile.h>
 
 #include "../app.h"
-#include "../auth.h"
 #include "../auth_int.h"
+#include "../gzochid-auth.h"
 #include "../scheme.h"
+#include "../scheme-task.h"
 #include "../task.h"
 
 #include "task.h"
@@ -130,7 +131,8 @@ SCM_DEFINE (primitive_cancel_task, "primitive-cancel-task", 1, 0, 0,
   return SCM_UNSPECIFIED;
 }
 
-void gzochid_api_task_init (void)
+void 
+gzochid_api_task_init (void)
 {
   SCM current_module = scm_current_module ();
   SCM gzochi_private_task = scm_c_resolve_module ("gzochi private task");
