@@ -1,5 +1,5 @@
 /* auth.c: Authorization management routines for gzochid
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2015 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,18 +23,14 @@
 #include <strings.h>
 
 #include "app.h"
-#include "auth.h"
 #include "auth_int.h"
 #include "game.h"
+#include "gzochid-auth.h"
 #include "log.h"
 #include "util.h"
 
 #define PLUGIN_INFO_FUNCTION "gzochid_auth_init_plugin"
 
-#ifndef GZOCHID_AUTH_PLUGIN_LOCATION
-#define GZOCHID_AUTH_PLUGIN_LOCATION "./auth"
-#endif /* GZOCHID_AUTH_PLUGIN_LOCATION */
-
 GQuark gzochid_auth_plugin_error_quark (void)
 {
   return g_quark_from_static_string ("gzochid-auth-plugin-error-quark");
@@ -155,8 +151,25 @@ static void probe_auth_plugins
       GSequence *plugin_queue = g_sequence_new (free);
       while ((file = g_dir_read_name (plugin_dir)) != NULL)
 	{
-	  gchar *stripped_file = remove_extension (file);
-	  gchar *path = g_build_filename (search_path, stripped_file, NULL);
+	  char *stripped_file = NULL;
+	  gchar *path = g_build_filename (search_path, file, NULL);
+
+	  /* Make a reasonable attempt to ignore non-regular files, like
+	     directories. This test is obviously not synchronous, which means
+	     that it could be "fooled" by a file that is removed or replaced
+	     with some other entity; the consequence is warning downstream. */
+
+	  if (!g_file_test (path, G_FILE_TEST_IS_REGULAR))
+	    {
+	      g_free (path);
+	      continue;
+	    }
+
+	  g_free (path);
+	  stripped_file = remove_extension (file);
+	  path = g_build_filename (search_path, stripped_file, NULL);
+
+	  free (stripped_file);
 
 	  if (g_sequence_lookup 
 	      (plugin_queue, path, gzochid_util_string_data_compare, NULL) 
@@ -172,9 +185,7 @@ static void probe_auth_plugins
 
 void gzochid_auth_init (gzochid_game_context *context)
 {
-  char *plugin_search_path = GZOCHID_AUTH_PLUGIN_LOCATION;
-
   if (g_module_supported ())
-    probe_auth_plugins (context, plugin_search_path);
+    probe_auth_plugins (context, context->auth_plugin_dir);
   else gzochid_info ("Plugins not supported; skipping auth plugin probe.");
 }
