@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2015 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <syslog.h>
 
 #include "app.h"
 #include "auth_int.h"
@@ -32,9 +31,13 @@
 #include "descriptor.h"
 #include "fsm.h"
 #include "game.h"
+#include "gzochid-storage.h"
 #include "log.h"
 #include "scheme.h"
+#include "scheme-task.h"
 #include "socket.h"
+#include "storage.h"
+#include "storage-mem.h"
 #include "task.h"
 #include "threads.h"
 
@@ -45,44 +48,62 @@
 #define SERVER_FS_APPS_DEFAULT "/var/gzochid/deploy"
 #define SERVER_FS_DATA_DEFAULT "/var/gzochid/data"
 
-static void initialize_application 
-(gzochid_game_context *context, gzochid_application_descriptor *descriptor)
+#ifndef GZOCHID_AUTH_PLUGIN_DIR
+#define GZOCHID_AUTH_PLUGIN_DIR "./auth"
+#endif /* GZOCHID_AUTH_PLUGIN_DIR */
+
+#ifndef GZOCHID_STORAGE_ENGINE_DIR
+#define GZOCHID_STORAGE_ENGINE_DIR "./storage"
+#endif /* GZOCHID_STORAGE_ENGINE_DIR */
+
+static void 
+initialize_application (gzochid_game_context *context, const char *dir,
+			gzochid_application_descriptor *descriptor)
 {
   gzochid_application_context *application_context =
     gzochid_application_context_new ();
   
+  application_context->deployment_root = strdup (dir);
+  application_context->load_paths = g_list_prepend 
+    (g_list_copy (descriptor->load_paths), strdup (dir));
+
   gzochid_application_context_init
     (application_context, (gzochid_context *) context, descriptor);
   g_source_attach ((GSource *) application_context->event_source, 
 		   g_main_loop_get_context (context->event_loop));
 }
 
-static void scan_app_dir (gzochid_game_context *context, char *dir)
+static void 
+scan_app_dir (gzochid_game_context *context, const char *dir)
 {
-  char *descriptor_file = g_strconcat (dir, "/", GAME_DESCRIPTOR_XML, NULL);
+  FILE *descriptor_file = NULL;
+  char *descriptor_filename = g_strconcat (dir, "/", GAME_DESCRIPTOR_XML, NULL);
 
   gzochid_application_descriptor *descriptor = NULL;
 
-  if (!g_file_test (descriptor_file, G_FILE_TEST_IS_REGULAR))
+  if (!g_file_test (descriptor_filename, G_FILE_TEST_IS_REGULAR))
     {
       gzochid_warning 
-	("%s does not exist or is not a regular file.", descriptor_file);
-      g_free (descriptor_file);
+	("%s does not exist or is not a regular file.", descriptor_filename);
+      g_free (descriptor_filename);
       return;
     }
 
+  descriptor_file = fopen (descriptor_filename, "r");
   descriptor = gzochid_config_parse_application_descriptor (descriptor_file);
+  fclose (descriptor_file);
 
   if (g_hash_table_contains (context->applications, descriptor->name))
     gzochid_warning
       ("Application in %s with name '%s' already exists; skipping.", 
        descriptor_file, descriptor->name);
-  else initialize_application (context, descriptor);
+  else initialize_application (context, dir, descriptor);
 
-  g_free (descriptor_file);
+  g_free (descriptor_filename);
 }
 
-static void scan_apps_dir (gzochid_game_context *context)
+static void 
+scan_apps_dir (gzochid_game_context *context)
 {
   GDir *dir = g_dir_open (context->apps_dir, O_RDONLY, NULL);
   const char *name = g_dir_read_name (dir);
@@ -98,22 +119,23 @@ static void scan_apps_dir (gzochid_game_context *context)
   g_dir_close (dir);
 }
 
-static void initialize_async (gpointer data, gpointer user_data)
+static void 
+initialize_async (gpointer data, gpointer user_data)
 {
-  gzochid_context *context = (gzochid_context *) user_data;
+  gzochid_context *context = user_data;
   gzochid_fsm_to_state (context->fsm, GZOCHID_GAME_STATE_RUNNING);
 }
 
-static void initialize_auth (int from_state, int to_state, gpointer user_data)
+static void 
+initialize_auth (int from_state, int to_state, gpointer user_data)
 {
-  gzochid_game_context *context = (gzochid_game_context *) user_data;
-
-  gzochid_auth_init (context);
+  gzochid_auth_init (user_data);
 }
 
-static void initialize_server (int from_state, int to_state, gpointer user_data)
+static void 
+initialize_server (int from_state, int to_state, gpointer user_data)
 {
-  gzochid_game_context *context = (gzochid_game_context *) user_data;
+  gzochid_game_context *context = user_data;
 
   context->server = gzochid_socket_server_context_new ();
   gzochid_socket_server_context_init 
@@ -122,9 +144,10 @@ static void initialize_server (int from_state, int to_state, gpointer user_data)
     ((gzochid_context *) context->server, GZOCHID_SOCKET_SERVER_STATE_RUNNING);
 }
 
-static void initialize_apps (int from_state, int to_state, gpointer user_data)
+static void 
+initialize_apps (int from_state, int to_state, gpointer user_data)
 {
-  gzochid_game_context *context = (gzochid_game_context *) user_data;
+  gzochid_game_context *context = user_data;
   
   if (!g_file_test (context->work_dir, G_FILE_TEST_EXISTS))
     {
@@ -146,57 +169,64 @@ static void initialize_apps (int from_state, int to_state, gpointer user_data)
   scan_apps_dir (context);
 }
 
-static void initialize_scheme_api
-(int from_state, int to_state, gpointer user_data)
+static void
+initialize_scheme_api (int from_state, int to_state, gpointer user_data)
 {
   gzochid_scheme_initialize_bindings ();
+  gzochid_scheme_task_initialize_bindings ();
 }
 
-static void initialize_serialization_registry
-(int from_state, int to_state, gpointer user_data)
+static void 
+initialize_serialization_registry (int from_state, int to_state, 
+				   gpointer user_data)
 {
   gzochid_task_initialize_serialization_registry ();
 }
 
 static void 
-initialize_scheme_task_serialization 
-(int from_state, int to_staet, gpointer user_data)
+initialize_scheme_task_serialization (int from_state, int to_state,
+				      gpointer user_data)
 {
   gzochid_task_register_serialization (&gzochid_scheme_task_serialization);
 }
 
-static void initialize_client_received_message_task_serialization
-(int from_state, int to_state, gpointer user_data)
+static void 
+initialize_client_received_message_task_serialization (int from_state, 
+						       int to_state, 
+						       gpointer user_data)
 {
   gzochid_register_client_received_message_task_serialization ();
 }
 
-static void initialize_complete
-(int from_state, int to_state, gpointer user_data)
+static void 
+initialize_complete (int from_state, int to_state, gpointer user_data)
 {
-  gzochid_game_context *context = (gzochid_game_context *) user_data;
+  gzochid_game_context *context = user_data;
   gzochid_thread_pool_push (context->pool, initialize_async, NULL, NULL);  
 }
 
-static gpointer event_loop (gpointer data)
+static gpointer 
+event_loop (gpointer data)
 {
-  gzochid_game_context *context = (gzochid_game_context *) data;
+  gzochid_game_context *context = data;
   g_main_loop_run (context->event_loop);
   return NULL;
 }
 
-static void initialize_event_loop 
-(int from_state, int to_state, gpointer user_data)
+static void 
+initialize_event_loop (int from_state, int to_state, gpointer user_data)
 {
   g_thread_new ("event-loop", event_loop, user_data);
 }
 
-gzochid_game_context *gzochid_game_context_new (void)
+gzochid_game_context *
+gzochid_game_context_new (void)
 {
   return calloc (1, sizeof (gzochid_game_context));
 }
 
-void gzochid_game_context_free (gzochid_game_context *context)
+void 
+gzochid_game_context_free (gzochid_game_context *context)
 {
   gzochid_context_free ((gzochid_context *) context);
 
@@ -206,8 +236,9 @@ void gzochid_game_context_free (gzochid_game_context *context)
   free (context);
 } 
 
-void gzochid_game_context_init
-(gzochid_game_context *context, gzochid_context *parent, GHashTable *config)
+void 
+gzochid_game_context_init (gzochid_game_context *context, 
+			   gzochid_context *parent, GHashTable *config)
 {
   long tx_timeout_ms = 0;
   gzochid_fsm *fsm = gzochid_fsm_new 
@@ -239,6 +270,42 @@ void gzochid_game_context_init
     context->work_dir = strdup (g_hash_table_lookup (config, "server.fs.data"));
   else context->work_dir = strdup (SERVER_FS_DATA_DEFAULT);
 
+  if (g_hash_table_contains (config, "server.auth.plugin.dir"))
+    context->auth_plugin_dir = 
+      strdup (g_hash_table_lookup (config, "server.auth.plugin.dir"));
+  else context->auth_plugin_dir = GZOCHID_AUTH_PLUGIN_DIR;
+
+  if (g_hash_table_contains (config, "server.storage.engine"))
+    {
+      char *dir = NULL;
+      char *env = getenv ("GZOCHID_STORAGE_ENGINE_DIR");
+      
+      if (env != NULL)
+	dir = env;
+      else 
+	{
+	  char *conf_dir = g_hash_table_lookup 
+	    (config, "server.storage.engine.dir");
+	  dir = conf_dir == NULL ? GZOCHID_STORAGE_ENGINE_DIR : conf_dir;
+	}
+
+      context->storage_engine = gzochid_storage_load_engine 
+	(dir, g_hash_table_lookup (config, "server.storage.engine"));
+    }
+  else gzochid_info 
+	 ("No durable storage engine configured; memory engine will be used.");
+
+  if (context->storage_engine == NULL)
+    {
+      gzochid_info ("\
+Using in-memory storage for application data. THIS CONFIGURATION IS NOT SAFE \
+FOR PRODUCTION USE.");
+
+      context->storage_engine = calloc (1, sizeof (gzochid_storage_engine));
+      context->storage_engine->interface = 
+	&gzochid_storage_engine_interface_mem;
+    }
+
   tx_timeout_ms = gzochid_config_to_long 
     (g_hash_table_lookup (config, "tx.timeout"), DEFAULT_TX_TIMEOUT_MS);
   context->tx_timeout.tv_sec = tx_timeout_ms / 1000;
@@ -277,25 +344,30 @@ void gzochid_game_context_init
   gzochid_context_init ((gzochid_context *) context, parent, fsm);
 }
 
-void gzochid_game_context_register_application
-(gzochid_game_context *context, char *name, gzochid_application_context *app)
+void 
+gzochid_game_context_register_application (gzochid_game_context *context, 
+					   char *name, 
+					   gzochid_application_context *app)
 {
   g_hash_table_insert (context->applications, name, app);
 }
 
-void gzochid_game_context_unregister_application
-(gzochid_game_context *context, char *name)
+void 
+gzochid_game_context_unregister_application (gzochid_game_context *context, 
+					     char *name)
 {
   g_hash_table_remove (context->applications, name);
 }
 
-gzochid_application_context *gzochid_game_context_lookup_application 
-(gzochid_game_context *context, char *name)
+gzochid_application_context *
+gzochid_game_context_lookup_application (gzochid_game_context *context, 
+					 char *name)
 {
   return g_hash_table_lookup (context->applications, name);
 }
 
-GList *gzochid_game_context_get_applications (gzochid_game_context *context)
+GList *
+gzochid_game_context_get_applications (gzochid_game_context *context)
 {
   return g_hash_table_get_values (context->applications);
 }
