@@ -1,5 +1,5 @@
 /* gzochid-migrate.c: Utility for migrating game data schema
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2015 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -33,10 +33,12 @@
 #include "config.h"
 #include "data.h"
 #include "descriptor.h"
+#include "game.h"
 #include "guile.h"
+#include "gzochid-storage.h"
 #include "reloc.h"
 #include "scheme.h"
-#include "storage.h"
+#include "scheme-task.h"
 #include "toollib.h"
 #include "tx.h"
 
@@ -109,9 +111,8 @@ struct migration_descriptor
  */
 
 static const gchar *
-attribute_value 
-(const gchar **attribute_names, const gchar **attribute_values,
- const gchar *attribute_name)
+attribute_value (const gchar **attribute_names, const gchar **attribute_values,
+		 const gchar *attribute_name)
 {
   int i = 0;
   while (attribute_names[i] != NULL)
@@ -125,15 +126,14 @@ attribute_value
 }
 
 static void
-start_element 
-(GMarkupParseContext *context, const gchar *element_name,
- const gchar **attribute_names, const gchar **attribute_values,
- gpointer user_data, GError **error)
+start_element (GMarkupParseContext *context, const gchar *element_name,
+	       const gchar **attribute_names, const gchar **attribute_values,
+	       gpointer user_data, GError **error)
 {
-  struct migration_descriptor *md = (struct migration_descriptor *) user_data;
+  struct migration_descriptor *md = user_data;
   const GSList *stack = g_markup_parse_context_get_element_stack (context);
   char *parent = stack == NULL || stack->next == NULL 
-    ? NULL : (char *) stack->next->data;
+    ? NULL : stack->next->data;
 
   if (strcmp (element_name, "migration") == 0)
     {
@@ -250,13 +250,12 @@ start_element
 }
 
 static void
-text 
-(GMarkupParseContext *context, const gchar *text, gsize text_len,
- gpointer user_data, GError **error)
+text (GMarkupParseContext *context, const gchar *text, gsize text_len,
+      gpointer user_data, GError **error)
 {
-  struct migration_descriptor *md = (struct migration_descriptor *) user_data;
+  struct migration_descriptor *md = user_data;
   const GSList *stack = g_markup_parse_context_get_element_stack (context);
-  char *parent = stack == NULL ? NULL : (char *) stack->data;
+  char *parent = stack == NULL ? NULL : stack->data;
  
   gchar *text_clone = g_strdup (text);
   text_clone = g_strstrip (text_clone);
@@ -275,8 +274,7 @@ text
 }
 
 static void
-error
-(GMarkupParseContext *context, GError *err, gpointer user_data)
+error (GMarkupParseContext *context, GError *err, gpointer user_data)
 {
   g_critical ("Failed to parse migration descriptor: %s", err->message);
   exit (EXIT_FAILURE);
@@ -308,7 +306,7 @@ static SCM
 enqueue_oids (SCM migration_ptr, SCM oids)
 {
   mpz_t oid;
-  struct migration *m = (struct migration *) scm_to_pointer (migration_ptr);
+  struct migration *m = scm_to_pointer (migration_ptr);
 
   mpz_init (oid);
 
@@ -351,9 +349,10 @@ initialize_scheme_bindings ()
 }
 
 static gzochid_storage_store *
-create_store (gzochid_storage_context *context, char *path)
+create_store (gzochid_storage_engine_interface *iface, 
+	      gzochid_storage_context *context, char *path)
 {
-  gzochid_storage_store *store = gzochid_storage_open 
+  gzochid_storage_store *store = iface->open 
     (context, path, GZOCHID_STORAGE_CREATE | GZOCHID_STORAGE_EXCL);
 
   if (store == NULL)
@@ -365,23 +364,25 @@ create_store (gzochid_storage_context *context, char *path)
 }
 
 static gzochid_storage_store *
-create_oid_scratch_storage (char *scratch_dir)
+create_oid_scratch_storage (gzochid_storage_engine_interface *iface, 
+			    char *scratch_dir)
 {
   gchar *filename = g_strconcat (scratch_dir, "/oids", NULL);
-  gzochid_storage_context *context = gzochid_storage_initialize (scratch_dir);
-  gzochid_storage_store *store = create_store (context, filename);
+  gzochid_storage_context *context = iface->initialize (scratch_dir);
+  gzochid_storage_store *store = create_store (iface, context, filename);
 
   g_free (filename);
   return store;
 }
 
 static void
-cleanup_oids_scratch_storage (gzochid_storage_store *s, char *scratch_dir)
+cleanup_oids_scratch_storage (gzochid_storage_engine_interface *iface, 
+			      gzochid_storage_store *s, char *scratch_dir)
 {
   gzochid_storage_context *context = s->context;
 
-  gzochid_storage_close (s);
-  gzochid_storage_destroy (context, scratch_dir);
+  iface->close_store (s);
+  iface->destroy_store (context, scratch_dir);
 }
 
 static SCM 
@@ -404,15 +405,15 @@ append_load_paths (GList *load_paths)
   GList *load_path_ptr = load_paths;
   while (load_path_ptr != NULL)
     {
-      gzochid_guile_add_to_load_path ((char *) load_path_ptr->data);
+      gzochid_guile_add_to_load_path (load_path_ptr->data);
       load_path_ptr = load_path_ptr->next;
     }
 }
 
 static struct migration *
-create_migration 
-(gzochid_application_context *context, struct migration_descriptor *md, 
- gboolean dry_run, gboolean quiet)
+create_migration (gzochid_application_context *context, 
+		  struct migration_descriptor *md, gboolean dry_run, 
+		  gboolean quiet)
 { 
   struct migration *m = malloc (sizeof (struct migration));
   char *scratch_dir_template = strdup ("gzochid-migrate-XXXXXXX");
@@ -428,7 +429,7 @@ create_migration
   assert (md->callback_name != NULL && strlen (md->callback_name) > 0);
   assert (md->callback_module != NULL && strlen (md->callback_module) > 0);
 
-  append_load_paths (context->descriptor->load_paths);
+  append_load_paths (context->load_paths);
   append_load_paths (md->load_paths);
 
   m->context = context;
@@ -438,7 +439,8 @@ create_migration
   m->quiet = quiet;
   m->pending_oids = g_queue_new ();
   
-  m->visited_oids = create_oid_scratch_storage (scratch_dir);
+  m->visited_oids = create_oid_scratch_storage 
+    (APP_STORAGE_INTERFACE (context), scratch_dir);
   m->visited_oids_path = scratch_dir;
 
   m->callback = resolve_or_die (md->callback_module, md->callback_name);
@@ -475,9 +477,11 @@ create_migration
 static void 
 cleanup_application_context (gzochid_application_context *context)
 {
-  gzochid_storage_close (context->meta);
-  gzochid_storage_close (context->names);
-  gzochid_storage_close (context->oids);
+  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (context);
+
+  iface->close_store (context->meta);
+  iface->close_store (context->names);
+  iface->close_store (context->oids);
 
   gzochid_application_context_free (context);
 }
@@ -485,9 +489,10 @@ cleanup_application_context (gzochid_application_context *context)
 static void
 cleanup_migration (struct migration *m)
 {
+  cleanup_oids_scratch_storage 
+    (APP_STORAGE_INTERFACE (m->context), m->visited_oids, m->visited_oids_path);
   cleanup_application_context (m->context);
 
-  cleanup_oids_scratch_storage (m->visited_oids, m->visited_oids_path);
   free (m->visited_oids_path);
 
   g_queue_free (m->pending_oids);
@@ -506,24 +511,36 @@ cleanup_migration (struct migration *m)
 }
 
 static gzochid_storage_store *
-open_store 
-(gzochid_storage_context *context, char *path, char *db)
+open_store (gzochid_application_context *context, char *path, char *db)
 {
   gchar *filename = g_strconcat (path, "/", db, NULL);
-  gzochid_storage_store *store = gzochid_tool_open_store (context, filename);
+  gzochid_storage_store *store = gzochid_tool_open_store 
+    (APP_STORAGE_INTERFACE (context), context->storage_context, filename);
 
   g_free (filename);
   return store;
 }
 
 static gzochid_application_context *
-create_application_context (char *gzochid_conf_path, char *app)
+create_application_context (char *path, char *app)
 {
-  GHashTable *config = gzochid_tool_load_game_config 
-    (gzochid_conf_path ? gzochid_conf_path : QUOTE (GZOCHID_CONF_LOCATION));
+  GHashTable *config = NULL; 
+  FILE *descriptor_file = NULL;
+
+  const char *env = getenv ("GZOCHID_CONF_LOCATION");
+  const char *gzochid_conf = NULL;
+
   gzochid_application_context *context = gzochid_application_context_new ();
   char *work_dir = NULL, *data_dir = NULL, *deploy_dir = NULL, *app_dir = NULL;
 
+  if (path != NULL)
+    gzochid_conf = path;
+  else if (env != NULL)
+    gzochid_conf = env;
+  else gzochid_conf = QUOTE (GZOCHID_CONF_LOCATION);
+
+  config = gzochid_tool_load_game_config (gzochid_conf);
+
   if (! g_hash_table_contains (config, "server.fs.data"))
     {
       g_critical ("server.fs.data must be set.");
@@ -535,7 +552,17 @@ create_application_context (char *gzochid_conf_path, char *app)
   deploy_dir = strdup (g_hash_table_lookup (config, "server.fs.apps"));
   app_dir = g_strconcat (deploy_dir, "/", app, "/", GAME_DESCRIPTOR_XML, NULL);
 
-  context->descriptor = gzochid_config_parse_application_descriptor (app_dir);
+  descriptor_file = fopen (app_dir, "r");
+  if (descriptor_file == NULL)
+    {
+      g_critical ("Failed to open application descriptor in %s", app_dir);
+      exit (EXIT_FAILURE);
+    }
+  
+  context->descriptor =
+    gzochid_config_parse_application_descriptor (descriptor_file);
+
+  fclose (descriptor_file);
 
   if (context->descriptor == NULL)
     {
@@ -543,7 +570,10 @@ create_application_context (char *gzochid_conf_path, char *app)
       exit (EXIT_FAILURE);
     }
 
-  context->storage_context = gzochid_storage_initialize (data_dir);
+  context->load_paths = g_list_prepend 
+    (g_list_copy (context->descriptor->load_paths), strdup (app_dir));
+  context->storage_context = 
+    APP_STORAGE_INTERFACE (context)->initialize (data_dir);
 
   if (context->storage_context == NULL)
     {
@@ -551,9 +581,9 @@ create_application_context (char *gzochid_conf_path, char *app)
       exit (EXIT_FAILURE);
     }
 
-  context->meta = open_store (context->storage_context, data_dir, "meta");
-  context->names = open_store (context->storage_context, data_dir, "names");
-  context->oids = open_store (context->storage_context, data_dir, "oids");
+  context->meta = open_store (context, data_dir, "meta");
+  context->names = open_store (context, data_dir, "names");
+  context->oids = open_store (context, data_dir, "oids");
 
   free (work_dir);
   free (data_dir);
@@ -568,8 +598,7 @@ invoke_callback (struct migration *m, gzochid_data_managed_reference *obj_ref)
 {
   SCM migration_ptr = scm_from_pointer (m, NULL);
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-  SCM obj = gzochid_scm_location_resolve 
-    (m->context, (gzochid_scm_location_info *) obj_ref->obj);
+  SCM obj = gzochid_scm_location_resolve (m->context, obj_ref->obj);
   SCM ret = SCM_BOOL_F;
 
   gpointer args[4];
@@ -609,9 +638,9 @@ migration_participant = { "migration", prepare, commit, rollback };
 static void
 migrate_object_tx (gpointer data)
 {
-  gpointer *args = (gpointer *) data;
-  struct migration *m = (struct migration *) args[0];
-  char *oid_str = (char *) args[1];
+  gpointer *args = data;
+  struct migration *m = args[0];
+  char *oid_str = args[1];
 
   mpz_t oid;
 
@@ -711,12 +740,13 @@ migrate_object (struct migration *m, char *oid_str)
   args[1] = oid_str;
 
   size_t oid_len = strlen (oid_str);
-  char *val = gzochid_storage_get 
+  char *val = APP_STORAGE_INTERFACE (m->context)->get 
     (m->visited_oids, oid_str, oid_len, NULL);
       
   if (val == NULL)
     {
-      gzochid_storage_put (m->visited_oids, oid_str, oid_len, "", 1);
+      APP_STORAGE_INTERFACE (m->context)->put
+	(m->visited_oids, oid_str, oid_len, "", 1);
       if (gzochid_transaction_execute (migrate_object_tx, args) 
 	  != GZOCHID_TRANSACTION_SUCCESS && !m->explicit_rollback)
 	{
@@ -738,11 +768,11 @@ static struct datum
 next_key (struct migration *m, struct datum last_key)
 {
   struct datum key = { NULL, 0 };
+  gzochid_storage_engine_interface *iface = APP_STORAGE_INTERFACE (m->context);
 
   if (last_key.data == NULL)
-    key.data = gzochid_storage_next_key 
-      (m->context->names, "o.", 2, &key.data_len);
-  else key.data = gzochid_storage_next_key 
+    key.data = iface->next_key (m->context->names, "o.", 2, &key.data_len);
+  else key.data = iface->next_key 
 	 (m->context->names, last_key.data, last_key.data_len, &key.data_len);
 
   if (key.data != NULL && strncmp ("o.", key.data, 2) != 0)
@@ -774,7 +804,7 @@ run_migration (struct migration *m)
 
 	  if (key.data == NULL)
 	    break;
-	  else oid_str = gzochid_storage_get 
+	  else oid_str = APP_STORAGE_INTERFACE (m->context)->get 
 		 (m->context->names, key.data, key.data_len, NULL);
 	}
 
@@ -867,7 +897,7 @@ Copyright (C) %s Julian Graham\n\
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
-          "2014");
+          "2015");
 }
 
 static void
