@@ -1,5 +1,5 @@
-/* io.h: Prototypes and declarations for serialization routines
- * Copyright (C) 2013 Julian Graham
+/* io.h: Prototypes and declarations for io.c
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -20,12 +20,24 @@
 
 #include <glib.h>
 
+#define GZOCHID_IO_ERROR gzochid_io_error_quark ()
+
+GQuark gzochid_io_error_quark (void);
+
+typedef enum
+  {
+    GZOCHID_IO_ERROR_SERIALIZATION
+  }
+  GzochidIoError;
+
 struct _gzochid_application_context;
 
 typedef struct _gzochid_io_serialization
 {
-  void (*serializer) (struct _gzochid_application_context *, void *, GString *);
-  void *(*deserializer) (struct _gzochid_application_context *, GString *);
+  void (*serializer) 
+    (struct _gzochid_application_context *, void *, GString *, GError **);
+  void *(*deserializer) 
+    (struct _gzochid_application_context *, GString *, GError **);
   void (*finalizer) (struct _gzochid_application_context *, void *);
 } gzochid_io_serialization;
 
