@@ -1,5 +1,5 @@
 ;; gzochi/private/reflect.scm: Exports for common binding reflection procedures
-;; Copyright (C) 2011 Julian Graham
+;; Copyright (C) 2013 Julian Graham
 ;;
 ;; gzochi is free software: you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -20,24 +20,37 @@
   (export gzochi:resolve-procedure)
   (import (guile)
 	  (ice-9 format)
+	  (ice-9 threads)
 	  (rnrs conditions)
 	  (rnrs exceptions))
 
-  (define (gzochi:resolve-procedure procedure module-name)
-    (let ((module (resolve-module module-name)))
-      (or module
-	  (raise 
-	   (condition 
-	    (make-assertion-violation)
-	    (make-message-condition 
-	     (format #f "Unable to resolve ~A" module-name)))))
+  (define module-resolution-mutex (make-mutex))
 
-      (let ((variable (module-variable module procedure)))
-	(or variable 
+  (define (gzochi:resolve-procedure procedure module-name)
+    (with-mutex module-resolution-mutex
+
+      ;; resolve-module is not thread-safe, at least as of Guile 2.0.6.
+      ;; If multiple threads attempt to resolve the same module name at once,
+      ;; some of them may get false negatives. Protecting this function with a
+      ;; a mutex seems to prevent that from happening, but there are surely
+      ;; performance implications.
+
+      (let ((module (resolve-module module-name #:ensure #f)))
+	(or module
 	    (raise 
 	     (condition 
 	      (make-assertion-violation)
 	      (make-message-condition 
-	       (format #f "No binding for ~A in ~A" procedure module-name)))))
-	(variable-ref variable))))
+	       (format #f "Unable to resolve ~A" module-name)))))
+
+	(let ((variable (module-variable module procedure)))
+	  (or variable 
+	      (raise 
+	       (condition 
+		(make-assertion-violation)
+		(make-message-condition 
+		 (format #f "No binding for ~A in ~A" 
+			 procedure module-name)))))
+
+	(variable-ref variable)))))
 )
