@@ -1,5 +1,5 @@
 /* session.c: Client session management routines for gzochid
- * Copyright (C) 2014 Julian Graham
+ * Copyright (C) 2015 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -23,15 +23,15 @@
 #include <string.h>
 
 #include "app.h"
-#include "auth.h"
 #include "auth_int.h"
 #include "data.h"
 #include "game.h"
+#include "gzochid-auth.h"
 #include "io.h"
 #include "log.h"
 #include "protocol.h"
 #include "reloc.h"
-#include "scheme.h"
+#include "scheme-task.h"
 #include "session.h"
 #include "task.h"
 #include "tx.h"
@@ -48,21 +48,27 @@ enum gzochid_client_session_operation
     GZOCHID_CLIENT_SESSION_OP_MESSAGE
   };
 
-typedef struct _gzochid_client_session_pending_operation
+struct _gzochid_client_session_pending_operation
 {
   enum gzochid_client_session_operation type;
   mpz_t target_session;
-} gzochid_client_session_pending_operation;
+};
 
-typedef struct _gzochid_client_session_pending_message_operation
+typedef struct _gzochid_client_session_pending_operation
+gzochid_client_session_pending_operation;
+
+struct _gzochid_client_session_pending_message_operation
 {
   gzochid_client_session_pending_operation base;
 
   unsigned char *message;
   short len;
-} gzochid_client_session_pending_message_operation;
+};
 
-typedef struct _gzochid_client_session_transaction_context
+typedef struct _gzochid_client_session_pending_message_operation
+gzochid_client_session_pending_message_operation;
+
+struct _gzochid_client_session_transaction_context
 {
   gzochid_application_context *context;
   
@@ -70,10 +76,13 @@ typedef struct _gzochid_client_session_transaction_context
   gzochid_client_session_pending_operation *login_operation;
 
   GList *operations;
-} gzochid_client_session_transaction_context;
+};
 
-static gzochid_client_session_transaction_context *create_transaction_context
-(gzochid_application_context *context)
+typedef struct _gzochid_client_session_transaction_context
+gzochid_client_session_transaction_context;
+
+static gzochid_client_session_transaction_context *
+create_transaction_context (gzochid_application_context *context)
 {
   gzochid_client_session_transaction_context *tx_context = 
     calloc (1, sizeof (gzochid_client_session_transaction_context));
@@ -83,8 +92,8 @@ static gzochid_client_session_transaction_context *create_transaction_context
   return tx_context;
 }
 
-static gzochid_client_session_pending_operation *create_disconnect_operation
-(mpz_t target_session)
+static gzochid_client_session_pending_operation *
+create_disconnect_operation (mpz_t target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
@@ -96,8 +105,8 @@ static gzochid_client_session_pending_operation *create_disconnect_operation
   return op;
 }
 
-static gzochid_client_session_pending_operation *create_login_success_operation
-(mpz_t target_session)
+static gzochid_client_session_pending_operation *
+create_login_success_operation (mpz_t target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
@@ -109,8 +118,8 @@ static gzochid_client_session_pending_operation *create_login_success_operation
   return op;  
 }
 
-static gzochid_client_session_pending_operation *create_login_failure_operation
-(mpz_t target_session)
+static gzochid_client_session_pending_operation *
+create_login_failure_operation (mpz_t target_session)
 {
   gzochid_client_session_pending_operation *op = 
     malloc (sizeof (gzochid_client_session_pending_operation));
@@ -122,8 +131,8 @@ static gzochid_client_session_pending_operation *create_login_failure_operation
   return op;  
 }
 
-static gzochid_client_session_pending_operation *create_message_operation
-(mpz_t target_session, unsigned char *msg, short len)
+static gzochid_client_session_pending_operation *
+create_message_operation (mpz_t target_session, unsigned char *msg, short len)
 {
   gzochid_client_session_pending_message_operation *msg_op = 
     malloc (sizeof (gzochid_client_session_pending_message_operation));
@@ -140,29 +149,31 @@ static gzochid_client_session_pending_operation *create_message_operation
   return op;
 }
 
-static int session_prepare (gpointer data)
+static int 
+session_prepare (gpointer data)
 {
   return TRUE;
 }
 
-static void free_operation (gpointer data)
+static void 
+free_operation (gpointer data)
 {
-  gzochid_client_session_pending_operation *op = 
-    (gzochid_client_session_pending_operation *) data;
+  gzochid_client_session_pending_operation *op = data;
 
   mpz_clear (op->target_session);
   free (op);
 }
 
-static void cleanup_transaction 
-(gzochid_client_session_transaction_context *tx_context)
+static void 
+cleanup_transaction (gzochid_client_session_transaction_context *tx_context)
 {
   g_list_free_full (tx_context->operations, free_operation);
   
   free (tx_context);
 }
 
-static void session_commit_operation
+static void 
+session_commit_operation
 (gzochid_client_session_transaction_context *tx_context, 
  gzochid_client_session_pending_operation *op)
 {
@@ -173,8 +184,7 @@ static void session_commit_operation
   gzochid_protocol_client *client = NULL;
 
   g_mutex_lock (&context->client_mapping_lock);
-  client = (gzochid_protocol_client *) g_hash_table_lookup 
-    (context->oids_to_clients, oid_str);
+  client = g_hash_table_lookup (context->oids_to_clients, oid_str);
 
   if (client == NULL)
     {
@@ -214,20 +224,19 @@ static void session_commit_operation
   g_mutex_unlock (&context->client_mapping_lock);
 }
 
-static void session_commit_operation_visitor (gpointer data, gpointer user_data)
+static void 
+session_commit_operation_visitor (gpointer data, gpointer user_data)
 {
-  gzochid_client_session_transaction_context *tx_context =
-    (gzochid_client_session_transaction_context *) user_data;
-  gzochid_client_session_pending_operation *op = 
-    (gzochid_client_session_pending_operation *) data;
+  gzochid_client_session_transaction_context *tx_context = user_data;
+  gzochid_client_session_pending_operation *op = data;
 
   session_commit_operation (tx_context, op);
 }
 
-static void session_commit (gpointer data)
+static void 
+session_commit (gpointer data)
 {
-  gzochid_client_session_transaction_context *tx_context = 
-    (gzochid_client_session_transaction_context *) data;
+  gzochid_client_session_transaction_context *tx_context = data;
 
   if (tx_context->login_operation != NULL)
     session_commit_operation (tx_context, tx_context->login_operation);
@@ -238,16 +247,17 @@ static void session_commit (gpointer data)
   cleanup_transaction (tx_context);
 }
 
-static void session_rollback (gpointer data)
+static void 
+session_rollback (gpointer data)
 {
-  gzochid_client_session_transaction_context *tx_context = 
-    (gzochid_client_session_transaction_context *) data;
+  gzochid_client_session_transaction_context *tx_context = data;
 
   cleanup_transaction (tx_context);
 }
 
-static gzochid_client_session_handler *deserialize_handler 
-(gzochid_application_context *context, GString *in, GError **err)
+static gzochid_client_session_handler *
+deserialize_handler (gzochid_application_context *context, GString *in, 
+		     GError **err)
 {
   gzochid_client_session_handler *handler = malloc 
     (sizeof (gzochid_client_session_handler)); 
@@ -260,8 +270,9 @@ static gzochid_client_session_handler *deserialize_handler
   return handler;
 }
 
-static gpointer deserialize_client_session
-(gzochid_application_context *context, GString *in, GError **err)
+static gpointer 
+deserialize_client_session (gzochid_application_context *context, GString *in, 
+			    GError **err)
 {
   gzochid_auth_identity *identity = 
     gzochid_auth_identity_deserializer (context, in, NULL);
@@ -280,9 +291,10 @@ static gpointer deserialize_client_session
   return session;
 }
 
-static void serialize_handler 
-(gzochid_application_context *context, gzochid_client_session_handler *handler,
- GString *out, GError **err)
+static void 
+serialize_handler (gzochid_application_context *context, 
+		   gzochid_client_session_handler *handler, GString *out, 
+		   GError **err)
 {
   gzochid_application_callback_serialization.serializer 
     (context, handler->received_message, out, NULL);
@@ -290,10 +302,11 @@ static void serialize_handler
     (context, handler->disconnected, out, NULL);
 }
 
-static void serialize_client_session 
-(gzochid_application_context *context, gpointer obj, GString *out, GError **err)
+static void 
+serialize_client_session (gzochid_application_context *context, gpointer obj, 
+			  GString *out, GError **err)
 {
-  gzochid_client_session *session = (gzochid_client_session *) obj;
+  gzochid_client_session *session = obj;
 
   gzochid_auth_identity_serializer (context, session->identity, out, NULL);
   gzochid_util_serialize_sequence 
@@ -308,8 +321,9 @@ static void serialize_client_session
   else gzochid_util_serialize_boolean (FALSE, out);
 }
 
-static void finalize_handler 
-(gzochid_application_context *context, gzochid_client_session_handler *handler)
+static void 
+finalize_handler (gzochid_application_context *context, 
+		  gzochid_client_session_handler *handler)
 {
   gzochid_application_callback_serialization.finalizer 
     (context, handler->received_message);
@@ -318,10 +332,10 @@ static void finalize_handler
   free (handler);
 }
 
-static void finalize_client_session
-(gzochid_application_context *context, gpointer obj)
+static void 
+finalize_client_session (gzochid_application_context *context, gpointer obj)
 {
-  gzochid_client_session *session = (gzochid_client_session *) obj;
+  gzochid_client_session *session = obj;
 
   gzochid_auth_identity_finalizer (context, session->identity);
   g_sequence_free (session->channels);
@@ -343,8 +357,9 @@ gzochid_io_serialization gzochid_client_session_serialization =
     finalize_client_session 
   };
 
-static void remove_session
-(gzochid_application_context *context, const char *oid_str, GError **err)
+static void 
+remove_session (gzochid_application_context *context, const char *oid_str, 
+		GError **err)
 {
   GError *local_err = NULL;
   GString *binding = g_string_new (SESSION_PREFIX);
@@ -365,8 +380,7 @@ static void remove_session
 
       if (local_err == NULL)
 	{
-	  gzochid_client_session *session = 
-	    (gzochid_client_session *) session_reference->obj;
+	  gzochid_client_session *session = session_reference->obj;
 	  gzochid_data_managed_reference *scm_session_reference =
 	    gzochid_data_create_reference_to_oid 
 	    (context, &gzochid_scm_location_aware_serialization, 
@@ -384,7 +398,7 @@ static void remove_session
 	      if (local_err->message != NULL)
 		gzochid_info 
 		  ("Unable to remove Scheme object for session '%s': %s",
-		   local_err->message);
+		   oid_str, local_err->message);
 
 	      g_clear_error (&local_err);
 	    }
@@ -402,15 +416,17 @@ static void remove_session
   g_string_free (binding, TRUE);
 }
 
-void gzochid_client_session_disconnected_worker
+void 
+gzochid_client_session_disconnected_worker 
 (gzochid_application_context *context, gzochid_auth_identity *identity,
  gpointer data)
 {
-  remove_session (context, (const char *) data, NULL);
+  remove_session (context, data, NULL);
+  free (data);
 }
 
-gzochid_client_session *gzochid_client_session_new 
-(gzochid_auth_identity *identity)
+gzochid_client_session *
+gzochid_client_session_new (gzochid_auth_identity *identity)
 {
   gzochid_client_session *session = (gzochid_client_session *)
     calloc (1, sizeof (gzochid_client_session));
@@ -422,15 +438,16 @@ gzochid_client_session *gzochid_client_session_new
   return session;
 }
 
-void gzochid_client_session_free (gzochid_client_session *session)
+void 
+gzochid_client_session_free (gzochid_client_session *session)
 {
   mpz_clear (session->scm_oid);
   g_sequence_free (session->channels);
   free (session);
 }
 
-static gzochid_client_session_transaction_context *join_transaction 
-(gzochid_application_context *context)
+static gzochid_client_session_transaction_context *
+join_transaction (gzochid_application_context *context)
 {
   gzochid_client_session_transaction_context *tx_context = NULL;
 
@@ -445,8 +462,9 @@ static gzochid_client_session_transaction_context *join_transaction
   return tx_context;
 }
 
-void gzochid_client_session_disconnect 
-(gzochid_application_context *context, gzochid_client_session *session)
+void 
+gzochid_client_session_disconnect (gzochid_application_context *context, 
+				   gzochid_client_session *session)
 {
   GError *err = NULL;
   char *oid_str = NULL;
@@ -469,8 +487,9 @@ void gzochid_client_session_disconnect
   free (oid_str);
 }
 
-void gzochid_client_session_send_login_success
-(gzochid_application_context *context, gzochid_client_session *session)
+void 
+gzochid_client_session_send_login_success (gzochid_application_context *context,
+					   gzochid_client_session *session)
 {
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
@@ -484,8 +503,9 @@ void gzochid_client_session_send_login_success
     (reference->oid);
 }
 
-void gzochid_client_session_send_login_failure
-(gzochid_application_context *context, gzochid_client_session *session)
+void 
+gzochid_client_session_send_login_failure (gzochid_application_context *context,
+					   gzochid_client_session *session)
 {
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
@@ -499,9 +519,10 @@ void gzochid_client_session_send_login_failure
   tx_context->login_failed = TRUE;
 }
 
-void gzochid_client_session_send_message 
-(gzochid_application_context *context, gzochid_client_session *session, 
- unsigned char *msg, short len)
+void 
+gzochid_client_session_send_message (gzochid_application_context *context, 
+				     gzochid_client_session *session, 
+				     unsigned char *msg, short len)
 {
   gzochid_client_session_transaction_context *tx_context =
     join_transaction (context);
@@ -515,24 +536,29 @@ void gzochid_client_session_send_message
      create_message_operation (reference->oid, msg, len));
 }
 
-typedef struct _gzochid_persistence_task_data
+struct _gzochid_persistence_task_data
 {
   gpointer data;
   gzochid_io_serialization *serialization;
   gzochid_oid_holder *holder;
-} gzochid_persistence_task_data;
+};
 
+typedef struct _gzochid_persistence_task_data gzochid_persistence_task_data;
 
-typedef struct _gzochid_prefix_binding_persistence_task_data
+struct _gzochid_prefix_binding_persistence_task_data
 {
   gzochid_persistence_task_data base;
 
   char *prefix;
-} gzochid_prefix_binding_persistence_task_data;
+};
 
-static void init_persistence_task 
-(gzochid_persistence_task_data *task, gzochid_io_serialization *serialization, 
- gpointer data, gzochid_oid_holder *holder)
+typedef struct _gzochid_prefix_binding_persistence_task_data
+ gzochid_prefix_binding_persistence_task_data;
+
+static void 
+init_persistence_task (gzochid_persistence_task_data *task, 
+		       gzochid_io_serialization *serialization, gpointer data, 
+		       gzochid_oid_holder *holder)
 {
   task->data = data;
   task->serialization = serialization;
@@ -554,18 +580,17 @@ gzochid_prefix_binding_persistence_task_data_new
   return task;
 }
 
-void gzochid_persistence_task_data_free (gzochid_persistence_task_data *task)
+void 
+gzochid_persistence_task_data_free (gzochid_persistence_task_data *task)
 {
   free (task);
 }
 
-static void persistence_task_worker
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
+static void 
+persistence_task_worker (gzochid_application_context *context, 
+			 gzochid_auth_identity *identity, gpointer data)
 {
-  gzochid_persistence_task_data *persistence_task = 
-    (gzochid_persistence_task_data *) data;
-
+  gzochid_persistence_task_data *persistence_task = data;
   gzochid_data_managed_reference *reference = 
     gzochid_data_create_reference 
     (context, persistence_task->serialization, persistence_task->data);
@@ -573,13 +598,13 @@ static void persistence_task_worker
   mpz_set (persistence_task->holder->oid, reference->oid);
 }
 
-static void prefix_binding_persistence_task_worker
-(gzochid_application_context *context, gzochid_auth_identity *identity, 
- gpointer data)
+static void 
+prefix_binding_persistence_task_worker (gzochid_application_context *context,
+					gzochid_auth_identity *identity, 
+					gpointer data)
 {
   gzochid_prefix_binding_persistence_task_data *
-    prefix_binding_persistence_task = 
-    (gzochid_prefix_binding_persistence_task_data *) data;
+    prefix_binding_persistence_task = data;
   gzochid_persistence_task_data *persistence_task =
     (gzochid_persistence_task_data *) prefix_binding_persistence_task;
   GString *binding = g_string_new (prefix_binding_persistence_task->prefix);
@@ -597,7 +622,8 @@ static void prefix_binding_persistence_task_worker
   g_string_free (binding, TRUE);
 }
 
-gzochid_task *gzochid_data_prefix_binding_persistence_task_new
+gzochid_task *
+gzochid_data_prefix_binding_persistence_task_new
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gzochid_io_serialization *serialization, gpointer data, 
  gzochid_oid_holder *holder, char *prefix)
@@ -619,16 +645,14 @@ gzochid_task *gzochid_data_prefix_binding_persistence_task_new
     (gzochid_application_task_thread_worker, application_task);
 }
 
-void gzochid_data_prefix_binding_persistence_task_free (gzochid_task *task)
+void 
+gzochid_data_prefix_binding_persistence_task_free (gzochid_task *task)
 {
-  gzochid_application_task *application_task = 
-    (gzochid_application_task *) task->data;
+  gzochid_application_task *application_task = task->data;
   gzochid_transactional_application_task_execution *execution = 
-    (gzochid_transactional_application_task_execution *) 
     application_task->data;
   gzochid_application_task *transactional_task = execution->task;
-  gzochid_persistence_task_data *data = 
-    (gzochid_persistence_task_data *) transactional_task->data;
+  gzochid_persistence_task_data *data = transactional_task->data;
   
   free (data);
   free (transactional_task);
@@ -637,9 +661,9 @@ void gzochid_data_prefix_binding_persistence_task_free (gzochid_task *task)
   free (task);
 }
 
-void gzochid_client_session_persist 
-(gzochid_application_context *context, gzochid_client_session *session, 
- mpz_t oid)
+void 
+gzochid_client_session_persist (gzochid_application_context *context, 
+				gzochid_client_session *session, mpz_t oid)
 {
   gzochid_game_context *game_context = 
     (gzochid_game_context *) ((gzochid_context *) context)->parent;
@@ -656,8 +680,9 @@ void gzochid_client_session_persist
   gzochid_data_prefix_binding_persistence_task_free (task);
 }
 
-void gzochid_sweep_client_sessions 
-(gzochid_application_context *context, gzochid_auth_identity *identity)
+void 
+gzochid_sweep_client_sessions (gzochid_application_context *context, 
+			       gzochid_auth_identity *identity)
 {
   mpz_t oid;
   GError *err = NULL;
